__all__ = ['NameVerDetectionError', 'Artifact', 'LocalArtifact', 'LocalRpmArtifact', 'RemoteArtifact']

import six.moves.urllib.parse
import itertools
import re
import os

class ArtifactError(Exception):
    pass


class NameVerDetectionError(ArtifactError):
    pass


class Artifact(object):
    """
    Generic class describing an artifact
    """
    def __init__(self, group, artifact='', version='', classifier='', extension=''):
        self.group = group
        self.artifact = artifact
        self.version = version
        self.classifier = classifier
        self.extension = extension

    def get_coordinates_string(self):
        return '{group}:{artifact}:{version}:{classifier}:{extension}'.format(group=self.group, artifact=self.artifact,
                                                                              version=self.version,
                                                                              classifier=self.classifier,
                                                                              extension=self.extension)

    def __repr__(self):
        return self.get_coordinates_string()


class LocalArtifact(Artifact):
    """
    Artifact for upload to repository
    """
    def __init__(self, group, local_path, artifact='', version='', classifier='', extension=''):
        self.local_path = local_path

        if not artifact and not version:
            artifact, version = self.detect_name_and_version()

        super(LocalArtifact, self).__init__(group=group, artifact=artifact, version=version, classifier=classifier,
                                            extension=extension)

    def detect_name_and_version(self):
        base_name = os.path.basename(self.local_path)
        result = re.match('^(?# name)(.*?)-(?=\d)(?# version)(\d.*)(?# extension)(\.[^.]+)$', base_name)

        if result is None:
            raise NameVerDetectionError('Automatic detection of name and/or version failed for %s', self.local_path)

        return result.group(1), result.group(2)


class LocalRpmArtifact(LocalArtifact):
    """
    Special case of local artifact, which can detect it's coordinates from RPM metadata
    """
    @staticmethod
    def get_artifact_group(url):
        if url is None:
            raise Exception('Web pages of the package not present in RPM metadata, please fill the URL tag in specfile')

        parts = six.moves.urllib.parse.urlsplit(url).netloc.split(".")
        return ".".join(itertools.ifilter(lambda x: x != "www", reversed(parts)))

    def __init__(self, local_path, group=None):
        try:
            import rpm
        except ImportError:
            raise ArtifactError("Can't import rpm module to detect name and version")
        ts = rpm.ts()
        fdno = os.open(local_path, os.O_RDONLY)
        headers = ts.hdrFromFdno(fdno)
        os.close(fdno)

        if not group:
            group = self.get_artifact_group(headers['url'])
        artifact = headers['name']
        version = '{v}-{r}'.format(v=headers['version'], r=headers['release'])
        super(LocalRpmArtifact, self).__init__(group=group, artifact=artifact, version=version, local_path=local_path)


class RemoteArtifact(Artifact):
    """
    Artifact in repository
    """
    def __init__(self, group=None, artifact='', version='', classifier='', extension='', url=None, repo_id=None):
        super(RemoteArtifact, self).__init__(group=group, artifact=artifact, version=version, classifier=classifier,
                                             extension=extension)
        self.repo_id = repo_id
        self.url = url

    @classmethod
    def from_repo_id_and_coordinates(cls, repo_id, coordinates):
        """

        :param repo_id:
        :param coordinates: e.g. 'com.fooware:foo:1.0.0'
        :return:
        """
        fields = coordinates.split(':')

        if len(fields) < 3:
            raise ArtifactError('Incorrect coordinates, at least group, artifact and version are obligatory')

        group, artifact, version = fields[0], fields[1], fields[2]

        classifier = extension = ''

        if len(fields) > 3:
            classifier = fields[3]

        if len(fields) > 4:
            extension = fields[4]

        return cls(group=group, artifact=artifact, version=version, classifier=classifier, extension=extension,
                   repo_id=repo_id)


"""
Contains classes for manipulating with a repository server
"""

__all__ = ['RepositoryClientError', 'WrongDataTypeError', 'ArtifactNotFoundError',
           'NexusRepositoryClient', 'NexusProRepositoryClient', 'repository_client_factory']

import requests
import logging
import os
import json
import base64

from repositorytools.lib.artifact import RemoteArtifact


logger = logging.getLogger(__name__)


class RepositoryClientError(Exception):
    """
    Base exception raised when working with NexusRepositoryClient and its descendants
    """
    pass

class WrongDataTypeError(RepositoryClientError):
    pass

class ArtifactNotFoundError(RepositoryClientError):
    pass

def repository_client_factory(*args, **kwargs):
    """
    Detects which kind of repository user wants to use and returns appropriate instance of it.

    :param args:
    :param kwargs:
    :return:
    """
    # short-term TODO: detect between Nexus and NexusPro
    # long-term TODO: detect and support also Artifactory and ArtifactoryPro
    return NexusProRepositoryClient(*args, **kwargs)


class NexusRepositoryClient(object):
    """
    Class for working with Sonatype Nexus OSS
    """
    DEFAULT_REPOSITORY_URL = 'https://repository'

    def __init__(self, repository_url=None, user=None, password=None, verify_ssl=True):
        """

        :param repository_url: url to repository server
        :param user: username for connecting to repository
        :param password: password for connecting to repository
        :param verify_ssl: False if you don't want to verify SSL certificate of the server
        :return:
        """
        self._verify_ssl = verify_ssl

        if repository_url:
            self._repository_url = repository_url
        else:
            self._repository_url = os.environ.get('REPOSITORY_URL', self.DEFAULT_REPOSITORY_URL)

        self._session = requests.session()

        if not user:
            user = os.environ.get('REPOSITORY_USER')

        if user:
            if not password:
                try:
                    password = os.environ['REPOSITORY_PASSWORD']
                except KeyError:
                    logger.error('Repository password not specified. Please specify repository password in environment'
                                 ' variable "REPOSITORY_PASSWORD"')
            self._session.auth = (user, password)

    def resolve_artifact(self, remote_artifact):
        data = self._send_json('service/local/artifact/maven/resolve', params=dict(g=remote_artifact.group,
                                                               a=remote_artifact.artifact,
                                                               v=remote_artifact.version,
                                                               r=remote_artifact.repo_id,
                                                               c=remote_artifact.classifier,
                                                               e=remote_artifact.extension))['data']

        remote_artifact.url = '{repository_url}/content/repositories/{repo}/{artifact_path}'.format(
            repository_url=self._repository_url, repo=remote_artifact.repo_id, artifact_path=data['repositoryPath'])

    def upload_artifacts(self, local_artifacts, repo_id, print_created_artifacts=True, _hostname_for_download=None,
                         _path_prefix='content/repositories'):
        """
        Uploads artifacts to repository.

        :param local_artifacts: list[LocalArtifact]
        :param repo_id: id of target repository
        :param print_created_artifacts: if True prints to stdout what was uploaded and where
        :return: list[RemoteArtifact]
        """

        # upload files
        remote_artifacts = []

        for local_artifact in local_artifacts:
            remote_artifact = self._upload_artifact(local_artifact=local_artifact, path_prefix=_path_prefix,
                                                    repo_id=repo_id, hostname_for_download=_hostname_for_download)
            remote_artifacts.append(remote_artifact)

        if print_created_artifacts:
            NexusRepositoryClient._print_created_artifacts(remote_artifacts, repo_id)

        return remote_artifacts

    def _upload_artifact(self, local_artifact, path_prefix, repo_id, hostname_for_download=None):

        filename = os.path.basename(local_artifact.local_path)
        logger.info('-> Uploading %s', filename)

        with open(local_artifact.local_path, 'rb') as f:
            headers = {'Content-Type': 'application/x-rpm'}

            # rgavf stands for repo-group-local_artifact-version-filename
            gavf = '{group}/{name}/{ver}/{filename}'.format(group=local_artifact.group.replace('.', '/'),
                                                            name=local_artifact.artifact, ver=local_artifact.version,
                                                            filename=filename)
            rgavf = '{repo_id}/{gavf}'.format(repo_id=repo_id, gavf=gavf)

            remote_path = '{path_prefix}/{rgavf}'.format(path_prefix=path_prefix, rgavf=rgavf)
            self._send(remote_path, method='POST', headers=headers, data=f)

        # if not specified, use repository url
        hostname_for_download = hostname_for_download or self._repository_url
        url = '{hostname}/content/repositories/{rgavf}'.format(hostname=hostname_for_download, rgavf=rgavf)

        # get classifier and extension from nexus
        path = 'service/local/repositories/{repo_id}/content/{gavf}?describe=maven2'.format(repo_id=repo_id, gavf=gavf)
        maven_metadata = self._send_json(path)['data']

        return RemoteArtifact(group=maven_metadata['groupId'], artifact=maven_metadata['artifactId'],
                              version=maven_metadata['version'], classifier=maven_metadata.get('classifier', ''),
                              extension=maven_metadata.get('extension', ''), url=url, repo_id=repo_id)

    def delete_artifact(self, url):
        """
        Deletes an artifact from repository.

        :param url: string
        :return:
        """
        r = self._session.delete(url)
        r.raise_for_status()

    @staticmethod
    def _print_created_artifacts(remote_artifacts, repo_id):
        caption = 'The following files were uploaded to repository {repo_id}'.format(repo_id=repo_id)

        if os.environ.get('TEAM_CITY_URL'):
            for remote_artifact in remote_artifacts:
                text = '<a href="{url}">{url}</a>'.format(url=remote_artifact.url)
                print("##teamcity[highlight title='{caption}' text='{text}']".format(caption=caption, text=text))

        else:
            print(caption)

            for remote_artifact in remote_artifacts:
                print(remote_artifact.url)

    def _send(self, path, method='GET', **kwargs):
        r = self._session.request(method, '{hostname}/{path}'.format(hostname=self._repository_url, path=path),
                                  verify=self._verify_ssl,
                                  **kwargs)

        logger.debug('response: %s', r.text)
        r.raise_for_status()

        return r

    def _send_json(self, path, json_data=None, method='GET', params=None):
        headers = {'Content-Type': 'application/json', 'accept': 'application/json'}
        if json_data is None:
            data = None
        else:
            data = json.dumps(json_data)
        r = self._send(path, data=data, headers=headers, method=method, params=params)

        if r.text:
            return json.loads(r.text)

    @staticmethod
    def _first_contains_second(first, second):
        """
        :param first dict
        :param second dict
        :return True if first has all keys from second and that they have same value
        """

        # to protect a user from hard-to-debug problems with incorrect data type
        # once I sent here a string with serialized dict and it took me hours to find the bug!
        if not isinstance(first, dict) or not isinstance(second, dict):
            raise WrongDataTypeError('Both arguments should be dict')

        result = all(k in first and first[k] == second[k] for k in second)
        return result


class NexusProRepositoryClient(NexusRepositoryClient):
    """
    Class for working with Sonatype Nexus Professional
    """
    def __init__(self, repository_url=None, user=None, password=None, verify_ssl=True, staging_repository_url=None):
        super(NexusProRepositoryClient, self).__init__(repository_url=repository_url, user=user, password=password,
                                                       verify_ssl=verify_ssl)

        """
        We redirect users to mirrors, but we don't mirror staging repositories, we when we upload artifacts and populate
        remote_url, we have to put there different alias of the repository server, which causes that they will not be
        redirected.
        """
        if staging_repository_url:
            self._staging_repository_url = staging_repository_url
        else:
            self._staging_repository_url = os.environ.get('STAGING_REPOSITORY_URL', self._repository_url)

    def upload_artifacts_to_staging(self, local_artifacts, repo_id, print_created_artifacts=True, upload_filelist=False):
        """
        :param local_artifacts: list[LocalArtifact]
        :param repo_id: name of staging repository
        :param print_created_artifacts: if True prints to stdout what was uploaded and where
        :param staging: bool
        :param upload_filelist: if True, creates and uploads a list of uploaded files

        :return: list[RemoteArtifact]
        """
        hostname_for_download = self._staging_repository_url
        path_prefix = 'service/local/staging/deployByRepositoryId'

        # upload files
        remote_artifacts = self.upload_artifacts(local_artifacts, repo_id, print_created_artifacts,
                                                 hostname_for_download, path_prefix)

        # upload filelist
        if upload_filelist:
            coord_list = [a.get_coordinates_string() for a in remote_artifacts]
            data = '\n'.join(coord_list)
            remote_path = '{path_prefix}/{repo_id}/{filelist_path}'.format(path_prefix=path_prefix, repo_id=repo_id,
                                                                           filelist_path=self._get_filelist_path(repo_id))
            self._send(remote_path, method='POST', data=data, headers={'Content-Type': 'text/csv'})

        return remote_artifacts

    def upload_artifacts_to_new_staging(self, local_artifacts, repo_id, print_created_artifacts=True,
                                        description='No description', upload_filelist=False):
        """
        Creates a staging repository in staging profile with name repo_id and uploads local_artifacts there.

        :param local_artifacts: list[LocalArtifact]
        :param repo_id: name of target repository
        :param print_created_artifacts: if True prints to stdout what was uploaded and where
        :param description: description of staging repo
        :param upload_filelist: see upload_artifacts_to_staging

        :return: list[RemoteArtifact]
        """
        repo_id = self.create_staging_repo(repo_id, description)
        remote_artifacts = self.upload_artifacts_to_staging(local_artifacts, repo_id, print_created_artifacts, upload_filelist)

        # close staging repo
        self.close_staging_repo(repo_id)
        return remote_artifacts

    @staticmethod
    def _get_filelist_path(repo_id):
        return '{repo_id}-filelist'.format(repo_id=repo_id)

    def get_artifact_metadata(self, remote_artifact):
        """
        Gets artifact's maven metadata.

        Metadata capability needs to be enabled to use this. Also indexing has to be enabled for that repo to make it
        work.

        :param remote_artifact:
        :return:
        """
        artifact_id = 'urn:maven/artifact#{coordinates}'.format(coordinates=remote_artifact.get_coordinates_string())
        logger.debug('artifact_id: %s', artifact_id)
        artifact_id_encoded = base64.b64encode(artifact_id)
        metadata_raw = self._send_json('service/local/index/custom_metadata/{repo_id}/{artifact_id_encoded}'.format(
            repo_id=remote_artifact.repo_id, artifact_id_encoded=artifact_id_encoded))

        metadata = {}

        for d in metadata_raw['data']:
            try:
                metadata[d["key"]] = d["value"]
            except KeyError:
                raise RepositoryClientError('Malformed artifact metadata. Missing key or value at artifact {atrifact}'.format(
                    artifact=remote_artifact
                ))

        return metadata

    def set_artifact_metadata(self, remote_artifact, metadata):
        """
        Sets artifact metadata.

        The same requirements as for get_artifact_metadata have to be met.

        :param remote_artifact:
        :param metadata: dict of keys and values you want to save there
        :return:
        """
        """
        check args
        """
        if not isinstance(metadata, dict):
            raise RepositoryClientError('Metadata has to be a dictionary')

        artifact_id = 'urn:maven/artifact#{coordinates}'.format(coordinates=remote_artifact.get_coordinates_string())
        logger.debug('artifact_id: %s', artifact_id)
        artifact_id_encoded = base64.b64encode(artifact_id)

        metadata_raw = []

        for key, value in metadata.iteritems():
            metadata_raw.append({"key": key, "value": value})

        return self._send_json('service/local/index/custom_metadata/{repo_id}/{artifact_id_encoded}'.format(
                               repo_id=remote_artifact.repo_id, artifact_id_encoded=artifact_id_encoded), method='POST',
                               json_data={"data": metadata_raw})

    def list_staging_repos(self, filter_dict=None):
        """

        :param filter_dict: dictionary with filters, for example {'description':'foo'}
        :return: list of dictionaries, each dict describes one staging repo
        """
        r = self._send_json('service/local/staging/profile_repositories')
        data = r['data']

        if not filter_dict:
            result = data
        else:
            result = [d for d in data if self._first_contains_second(d, filter_dict)]

        logger.debug('list_staging_repos result: %s', result)
        return result

    def create_staging_repo(self, profile_name, description):
        """
        Creates a staging repository
        :param profile_name: name of staging profile
        :param description: description of created staging repository
        :return: id of newly created staging repository
        """
        profile = self._get_staging_profile(profile_name)
        logger.info('Creating staged repo in profile %s, description: %s', profile_name, description)
        r = self._send_json('service/local/staging/profiles/{id}/start'.format(id=profile['id']),
                            {'data': {'description': description}}, method='POST')
        result = r['data']['stagedRepositoryId']
        logger.info('Created staged repo with ID %s', result)
        return result

    def close_staging_repo(self, repo_id, description=''):
        """
        Closes a staging repository. After close, no files can be added.

        :param repo_id: id of staging repository
        :param description: if specified, updates description of staged repository
        :return:
        """
        self.close_staging_repos([repo_id], description)

    def close_staging_repos(self, repo_ids, description=''):
        """
        Closes multiple staging repositories.

        :param repo_ids: list of repo IDs (strings)
        For description of other params see close_staging_repo.
        :return:
        """
        data = {'data': {'stagedRepositoryIds': repo_ids, 'description': description}}
        return self._send_json('service/local/staging/bulk/close', data, method='POST')

    def drop_staging_repo(self, repo_id, description='No description'):
        """
        Deletes a staging repository and all artifacts inside.

        :param repo_id: id of staging repository
        :return:
        """
        self.drop_staging_repos([repo_id], description=description)

    def drop_staging_repos(self, repo_ids, description='No description'):
        """
        Deletes multiple staging repositories.

        :param repo_ids: list of repo IDs (strings)
        :return:
        """
        data = {'data': {'stagedRepositoryIds': repo_ids, 'description': description}}
        return self._send_json('service/local/staging/bulk/drop', data, method='POST')

    def release_staging_repo(self, repo_id, description='No description', auto_drop_after_release=True,
                             keep_metadata=False):
        """
        Releases all contents of a staging repository to a release repository which this staging repository targets.

        :param repo_id: id of staging repository
        :param description:
        :param auto_drop_after_release: set this to True if you want to delete the staging repository after releasing
        :param keep_metadata: Keeps custom maven metadata of artifacts after release. Works only there is list of
         artifacts created by upload_artifacts_to_new_staging with upload_filelist=False. It is because current Nexus 2.x
         can't do keep the metadata after release, so we manually read the metadata, release and then set them again.
        :return:
        """
        if keep_metadata:
            # download list of artifacts
            resp = self._send('content/repositories/{repo_id}/{filelist_path}'.format(repo_id=repo_id,
                                                                                      filelist_path=self._get_filelist_path(repo_id)))

            artifacts = [RemoteArtifact.from_repo_id_and_coordinates(repo_id, coordinates=coords)
                        for coords in resp.text.split('\n')]

            # download metadata for all files
            for artifact in artifacts:
                artifact.metadata = self.get_artifact_metadata(artifact)

            release_repo_id = self._get_target_repository(repo_id)

        data = {'data': {'stagedRepositoryIds': [repo_id], 'description': description,
                         'autoDropAfterRelease': auto_drop_after_release}}
        result = self._send_json('service/local/staging/bulk/promote', data, method='POST')

        if keep_metadata:
            for artifact in artifacts:
                artifact.repo_id = release_repo_id
                self.set_artifact_metadata(artifact, artifact.metadata)

        return result

    def _get_staging_profile(self, name):
        staging_profiles = self._send_json('service/local/staging/profiles')

        for i in staging_profiles["data"]:
            if i["name"] == name:
                return i

        raise RepositoryClientError('No staging profile with name {name}'.format(name=name))

    def _get_target_repository(self, staging_repo_id):
        data = self._send_json('service/local/staging/repository/{staging_repo_id}'.format(staging_repo_id=staging_repo_id))
        return data['releaseRepositoryId']



__all__ = ['ArtifactCLI']

from ..common import CLI, logger
import argparse
import repositorytools
import json
import sys

class ArtifactCLI(CLI):
    def _get_parser(self):
        parser = argparse.ArgumentParser(description='A command line tool for working with artifacts')
        subparsers = parser.add_subparsers()

        # upload
        subparser = subparsers.add_parser('upload', help='Uploads an artifact to repository, can detect name and'
                                                             ' version from filename')
        subparser.add_argument("-s", "--staging", action="store_true", default=False,
                            help="Uploads to a newly created staging repository which targets given repository")
        subparser.add_argument("-x", "--use-existing", action="store_true", default=False,
                            help="To be used with -s, doesn't create a new repo, but uploads directly to an existing"
                                 "staging repo")
        subparser.add_argument("--upload-filelist", action="store_true", default=False, help="uploads list of uploaded "
                                                                                                 "files")
        subparser.add_argument("--artifact", help="name of artifact, if omitted, will be detected from filename")
        subparser.add_argument("--version", help="version of artifact, if omitted, will be detected from filename")
        subparser.add_argument("-d", "--description", dest="description", default='No description',
                                   help="Description of a staging repository")

        subparser.add_argument("local_file", help="path to an artifact on your machine")
        subparser.add_argument("repo_id", help="id of target repository")
        subparser.add_argument("group", help="artifact group")
        subparser.set_defaults(func=self.upload)

        # delete
        subparser = subparsers.add_parser('delete', help='Deletes an artifact from repository')
        subparser.add_argument("url", help="URL of the artifact")
        subparser.set_defaults(func=self.delete)

        # get metadata
        subparser = subparsers.add_parser('get-metadata', help="Prints artifact's metadata")
        subparser.add_argument("repo_id", help="id of repository containing the artifact")
        subparser.add_argument("coordinates", help="group:artifact:version[:classifier[:extension]]")
        subparser.set_defaults(func=self.get_metadata)

        # set metadata
        subparser = subparsers.add_parser('set-metadata', help="Sets artifact's metadata")
        subparser.add_argument("metadata", help="Dict in JSON format. All keys and values have to be strings,"
                                                          "e.g. '{\"key1\":\"value1\",\"key2\":\"value2\"}'")
        subparser.add_argument("repo_id", help="id of repository containing the artifact")
        subparser.add_argument("coordinates", help="group:artifact:version[:classifier[:extension]]", nargs='+')
        subparser.set_defaults(func=self.set_metadata)

        # resolve
        subparser = subparsers.add_parser('resolve', help="Resolves artifacts' URLs")
        subparser.add_argument("repo_id", help="id of repository containing the artifact")
        subparser.add_argument("coordinates", help="group:artifact:version[:classifier[:extension]]", nargs='+')
        subparser.set_defaults(func=self.resolve)
        return parser

    def resolve(self, args):
        artifacts = [ repositorytools.RemoteArtifact.from_repo_id_and_coordinates(args.repo_id, coordinates_item)
                      for coordinates_item in args.coordinates ]

        for artifact in artifacts:
            self.repository.resolve_artifact(artifact)

        output = '\n'.join(artifact.url for artifact in artifacts)
        print(output)
        return output

    def upload(self, args):
        try:
            artifact = repositorytools.LocalArtifact(local_path=args.local_file, group=args.group,
                                                     artifact=args.artifact, version=args.version)
        except repositorytools.NameVerDetectionError:
            logger.exception('Unable to create instance of artifact.')
            sys.exit(1)

        if args.staging:
            if not args.use_existing:
                return self.repository.upload_artifacts_to_new_staging([artifact], args.repo_id, True,
                                                                       description=args.description,
                                                                       upload_filelist=args.upload_filelist)
            else:
                return self.repository.upload_artifacts_to_staging([artifact], args.repo_id, True,
                                                                   upload_filelist=args.upload_filelist)
        else:
            return self.repository.upload_artifacts([artifact], args.repo_id, True)

    def delete(self, args):
        self.repository.delete_artifact(args.url)

    def get_metadata(self, args):
        artifact = repositorytools.RemoteArtifact.from_repo_id_and_coordinates(args.repo_id, args.coordinates)
        metadata = self.repository.get_artifact_metadata(artifact)
        output = json.dumps(metadata)
        print(output)
        return output

    def set_metadata(self, args):
        metadata = json.loads(args.metadata)
        for coordinates_item in args.coordinates:
            artifact = repositorytools.RemoteArtifact.from_repo_id_and_coordinates(args.repo_id, coordinates_item)
            self.repository.set_artifact_metadata(artifact, metadata)


from abc import ABCMeta, abstractmethod
import logging
from six.moves import http_client
import sys

import repositorytools

logger = logging.getLogger(sys.argv[0])


def configure_logging(debug):
    if debug:
        logging.basicConfig(level=logging.DEBUG)
        http_client.HTTPConnection.debuglevel = 1
        requests_log = logging.getLogger("requests.packages.urllib3")
        requests_log.setLevel(logging.INFO)
        requests_log.propagate = True
    else:
        logging.basicConfig(level=logging.INFO)


class CLI(object):
    """
    Base class for cli
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def _get_parser(self):
        """
        :return: argparse.ArgumentParser
        """
        pass

    def __init__(self):
        self.parser = self._get_parser()
        self.parser.add_argument("-D", "--debug", action="store_true", dest="debug", default=False,
                                 help="Print lots of debugging information")
        self.repository = None

    def run(self, args=None):
        args_namespace = self.parser.parse_args(args)
        configure_logging(args_namespace.debug)
        # in tests we don't use 'str(sys.argv), so we can log actual arguments
        used_args = args or sys.argv[1:]
        logger.info('Started %s, with arguments %s', sys.argv[0], str(used_args))

        """
        This runs the function that is assigned to the sub-command by calling of set_defaults
        """
        self.repository = repositorytools.repository_client_factory()
        return args_namespace.func(args_namespace)


#!/usr/bin/env python

from setuptools import setup, find_packages
import re
import sys
import os

version = ''
with open('repositorytools/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

install_requires=['requests>=1.1', 'six']

if sys.version_info < (2,7):
    install_requires.append("argparse < 2")

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

setup(name='repositorytools',
      version=version,
      description='Tools for working with artifact repositories',
      long_description=README,
      author='Michel Samia',
      author_email='stardust1985@gmail.com',
      url='https://github.com/stardust85/repositorytools',
      license='Apache 2.0',
      platforms='any',
      install_requires=install_requires,

      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Console',
      ],

      packages=find_packages(),
      scripts=['scripts/artifact', 'scripts/repo'],
      download_url='https://github.com/stardust85/repositorytools/tarball/{version}'.format(version=version)
)
